
ConfigTable::DESCRIPTER.concat [

  [ 'with',     [ '',
                  'name,name...',
                  'package names that you want to install',
                  'ALL' ] ],
  [ 'without',  [ '',
                  'name,name...',
                  'package names that you do not want to install',
                  'NONE' ] ]
]


class PackageManager < ToplevelInstaller

  def run_metaconfigs
    packages().each do |name|
      MetaConfigEnvironment.eval_file "#{srcdir_root}/packages/#{name}/#{metaconfig}"
    end
  end

  def print_usage( f )
    super
    f.puts 'Inluded packages:'
    f.puts '  ' + packages.sort.join(' ')
    f.puts
  end

  #
  # tasks
  #

  def exectask( task )
    load_installers
    init_selection
    super
  end

  def exec_config
    run_hook 'pre-config'
    each_selected_installers {|inst| inst.exec_config }
    @config.save
    run_hook 'post-config'
  end

  def exec_setup
    run_hook 'pre-setup'
    each_selected_installers {|inst| inst.exec_setup }
    run_hook 'post-setup'
  end

  def exec_install
    run_hook 'pre-install'
    each_selected_installers {|inst| inst.exec_install }
    run_hook 'post-install'
  end

  def exec_clean
    run_hook 'pre-clean'
    each_selected_installers {|inst| inst.exec_clean }
    run_hook 'post-clean'
  end

  #
  # packages
  #

  def packages
    packs = all_dirs(srcdir + '/packages')
    packs.empty? and raise InstallError, 'no package available'
    packs
  end

  def load_installers
    @installers = {}
    packages.each do |dir|
      @installers[dir] = Installer.new( @config, @options,
                                        srcdir + '/packages/' + dir,
                                        'packages/' + dir )
    end
  end

  def init_selection
    with    = extract_sel( config('with') )
    without = extract_sel( config('without') )

    @selected = @installers.keys.find_all {|name|
        (with.empty? or with.include? name) and
        not without.include? name
    }
  end

  def extract_sel( list )
    a = list.split(',')
    a.each do |name|
      @installers.key? name or raise InstallError, "no such package: #{name}"
    end
    a
  end

  def each_selected_installers
    dive_into( 'packages' ) {
        @selected.each do |name|
          dive_into( 'packages/' + name ) {
              yield @installers[name]
          }
        end
    }
  end

end

